# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/dmsSvn/DmsSvn.py $
# Last committed:     $Revision: 338329 $; $Author: svcdms $; $Date: 2019-09-30 16:27:04 +0200 (ma, 30 sep 2019) $
#
from svn import local, exception
from filecmp import cmp
from datetime import datetime
from os import path, makedirs
from re import sub


class DmsSvn:
    def __init__(self, folder_settings, logger, db, sensor):
        self._folder_settings = folder_settings
        self._logger = logger
        self._db = db
        self._sensor = sensor
        self.update_folder(sensor)

    def update_folder(self, subfolder):
        try:
            self._lc = local.LocalClient(self._folder_settings["primary_root"] + subfolder)
            self._lc.cleanup()
            self._lc.update()
        except (OSError, exception.SvnException):
            self.add_folders(subfolder)
            self.update_folder(self._folder_settings["primary_root"] + subfolder)

    def add_folders(self, subfolders):
        # try to add folders if necessary
        self._logger.info("DmsSvn.add_folders: Adding {}".format(subfolders))

        lpath = self._folder_settings["primary_root"] + '/' + sub('^/', '', subfolders)
        try:
            makedirs(lpath)
        except FileExistsError:
            pass

        l_subfolders = lpath.split('/')
        nsub = len(l_subfolders)
        while nsub >= 0:
            nsub = nsub - 1
            try:
                lc_up = local.LocalClient(lpath)
                break
            except OSError:
                # move upwards and find valid local svn client
                lpath = '/'.join(l_subfolders[0:nsub])
                pass

        lc_up.add(lpath)
        lc_up.commit("commit from automatic process")
        self._lc = local.LocalClient(lpath)

    def add_file(self, subfolders, file):
        # try to add folders if necessary
        self._logger.info("DmsSvn.add_file: Adding {}/{}".format(subfolders, file))
        subfolders = sub('^/', '', subfolders)
        try:
            self._lc.add(self._folder_settings["primary_root"] + '/' + subfolders + '/' + file)
        except exception.SvnException:
            return

    def commit(self):
        self._lc.commit("commit from automatic process")

    def update_secondary_root(self, remote_folder):
        self._logger.debug("DmsSvn.update_secondary_root({})".format(self._folder_settings["secondary_root"] + '/' + self._sensor))

        lpath = self._folder_settings["secondary_root"] + remote_folder
        l_subfolders = lpath.split('/')
        nsub = len(l_subfolders)
        while nsub >= 0:
            nsub = nsub - 1
            try:
                secondary_svn = local.LocalClient(lpath)
                break
            except OSError:
                # move upwards and find valid local svn client
                lpath = '/'.join(l_subfolders[0:nsub])
                pass

        try:
            secondary_svn.update()
            self._logger.debug("DmsSvn.update_secondary_root: secondary root updated")
        except exception.SvnException:
            self._logger.warn("DmsSvn.update_secondary_root: could not update secondary root; clean-up tried")
            try:
                secondary_svn.cleanup()
                self._logger.warn("DmsSvn.update_secondary_root: clean-up folder successful")
                secondary_svn.update()
                self._logger.warn("DmsSvn.update_secondary_root: update folder successful")
            except exception.SvnException:
                self._logger.error("DmsSvn.update_secondary_root: could not clean-up and update secondary root")
                raise

    # check for differences between the original file and the one in svn
    def no_diff(self, remote_folder, file):
        self._logger.debug("DmsSvn.no_diff({},{})".format(remote_folder, file))
        # check out to a secondary root
        self.update_secondary_root(remote_folder)

        primary_file = self._folder_settings["primary_root"] + remote_folder + '/' + file
        secondary_file = self._folder_settings["secondary_root"] + remote_folder + '/' + file

        # compare the primary and secondary file
        if cmp(primary_file, secondary_file, shallow="FALSE"):
            self._logger.debug("DmsSvn.no_diff: the files with name {} are identical".format(file))
            return True
        else:
            self._db.add_file_to_list(remote_folder, path.dirname(primary_file), file, 'svn_compare',
                                      datetime.now(), datetime.now(), 1)
            self._db.add_file_to_list(remote_folder, path.dirname(secondary_file), file, 'svn_compare',
                                      datetime.now(), datetime.now(), 1)
            self._logger.debug("DmsSvn.no_diff: we have a problem with svn {}".format(file))
            return False
